@@ -406,8 +406,12 @@ static int zone_status(zone_t *zone, ctl_args_t *args)
 				bool frozen = ufrozen && ufreeze_applies(i);
 				ret = snprintf(buff, sizeof(buff), frozen ? "frozen" : "pending");
 			} else {
-				ret = knot_time_print(TIME_PRINT_HUMAN_MIXED,
-				                      ev_time, buff, sizeof(buff));
+				knot_time_print_t format = TIME_PRINT_HUMAN_MIXED;
+				if (ctl_has_flag(args->data[KNOT_CTL_IDX_FLAGS],
+				                 CTL_FLAG_STATUS_UNIXTIME)) {
+					format = TIME_PRINT_UNIX;
+				}
+				ret = knot_time_print(format, ev_time, buff, sizeof(buff));
 			}
 			if (ret < 0 || ret >= sizeof(buff)) {
 				return KNOT_ESPACE;
@@ -516,7 +520,7 @@ static int init_backup(ctl_args_t *args, bool restore_mode)
 	}
 
 	// Evaluate filters (and possibly fail) before writing to the filesystem.
-	bool filter_zonefile, filter_journal, filter_timers, filter_kaspdb,
+	bool filter_zonefile, filter_journal, filter_timers, filter_kaspdb, filter_keysonly,
 	     filter_catalog, filter_quic;
 
 	// The default filter values are set just in this paragraph.
@@ -528,6 +532,8 @@ static int init_backup(ctl_args_t *args, bool restore_mode)
 	                          CTL_FILTER_BACKUP_TIMERS, CTL_FILTER_BACKUP_NOTIMERS) &&
 	    eval_opposite_filters(args, &filter_kaspdb, true,
 	                          CTL_FILTER_BACKUP_KASPDB, CTL_FILTER_BACKUP_NOKASPDB) &&
+	    eval_opposite_filters(args, &filter_keysonly, false,
+	                          CTL_FILTER_BACKUP_KEYSONLY, CTL_FILTER_BACKUP_NOKEYSONLY) &&
 	    eval_opposite_filters(args, &filter_catalog, true,
 	                          CTL_FILTER_BACKUP_CATALOG, CTL_FILTER_BACKUP_NOCATALOG) &&
 	    eval_opposite_filters(args, &filter_quic, false,
@@ -558,6 +564,7 @@ static int init_backup(ctl_args_t *args, bool restore_mode)
 	ctx->backup_journal = filter_journal;
 	ctx->backup_timers = filter_timers;
 	ctx->backup_kaspdb = filter_kaspdb;
+	ctx->backup_keysonly = filter_keysonly && !filter_kaspdb; // Priority of '+kaspdb'.
 	ctx->backup_catalog = filter_catalog;
 	ctx->backup_quic = filter_quic;
 
@@ -577,6 +584,8 @@ static int deinit_backup(ctl_args_t *args)
 	return zone_backup_deinit(latest_backup_ctx(args));
 }
 
+static int zone_keys_load(zone_t *zone, _unused_ ctl_args_t *args);
+
 static int zone_backup_cmd(zone_t *zone, ctl_args_t *args)
 {
 	zone_backup_ctx_t *ctx = latest_backup_ctx(args);
@@ -591,19 +600,41 @@ static int zone_backup_cmd(zone_t *zone, ctl_args_t *args)
 		return KNOT_EPROGRESS;
 	}
 
+	ctx->zone_count++;
+
+	int ret;
+	if (!ctx->backup_global) {
+		ret = global_backup(ctx, zone_catalog(zone), zone->name);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+	}
+
+	if (ctx->backup_keysonly) {
+		ret = zone_backup_keysonly(ctx, conf(), zone);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+
+		if (ctx->restore_mode) {
+			ret = zone_keys_load(zone, args);
+			if (ret != KNOT_EOK) {
+				return ret;
+			}
+		}
+
+		if (!ctx->backup_zonefile && !ctx->backup_journal && !ctx->backup_timers &&
+		    !ctx->backup_kaspdb && !ctx->backup_catalog && !ctx->backup_quic) {
+			return ret;
+		}
+	}
+
 	zone->backup_ctx = ctx;
 	pthread_mutex_lock(&ctx->readers_mutex);
 	ctx->readers++;
 	pthread_mutex_unlock(&ctx->readers_mutex);
-	ctx->zone_count++;
 
-	int ret = schedule_trigger(zone, args, ZONE_EVENT_BACKUP, true);
-
-	if (ret == KNOT_EOK && !ctx->backup_global && (ctx->restore_mode || !ctx->failed)) {
-		ret = global_backup(ctx, zone_catalog(zone), zone->name);
-	}
-
-	return ret;
+	return schedule_trigger(zone, args, ZONE_EVENT_BACKUP, true);
 }
 
 static int zones_apply_backup(ctl_args_t *args, bool restore_mode)
@@ -682,6 +713,11 @@ static int zone_keys_load(zone_t *zone, _unused_ ctl_args_t *args)
 		return KNOT_ENOTSUP;
 	}
 
+	if (zone->contents == NULL) {
+		log_zone_notice(zone->name, "zone is not loaded yet");
+		return KNOT_EOK;
+	}
+
 	return schedule_trigger(zone, args, ZONE_EVENT_DNSSEC, true);
 }
 
@@ -1581,8 +1617,8 @@ static int zone_purge(zone_t *zone, ctl_args_t *args)
 		}
 
 		// Expire the zone.
-		// KNOT_EOK is the only return value from event_expire().
-		(void)schedule_trigger(zone, args, ZONE_EVENT_EXPIRE, true);
+		// No ret, KNOT_EOK is the only return value from event_expire().
+		(void)zone_events_schedule_blocking(zone, ZONE_EVENT_EXPIRE, true);
 	}
 
 	purge_flag_t params =
