@@ -85,6 +85,7 @@ static int generate_dnssec_key(dnssec_keystore_t *keystore,
 	return KNOT_EOK;
 
 fail:
+	(void)dnssec_keystore_remove(keystore, *id);
 	dnssec_key_free(*key);
 	*key = NULL;
 	free(*id);
@@ -124,9 +125,6 @@ static int generate_keytag_unconflict(kdnssec_ctx_t *ctx,
 	}
 
 	for (size_t i = 0; i < GENERATE_KEYTAG_ATTEMPTS; i++) {
-		dnssec_key_free(*key);
-		free(*id);
-
 		int ret = generate_dnssec_key(ctx->keystore, ctx->zone->dname, label,
 		                              ctx->policy->algorithm, size, flags,
 		                              id, key);
@@ -136,6 +134,10 @@ static int generate_keytag_unconflict(kdnssec_ctx_t *ctx,
 		if (!keytag_in_use(ctx, dnssec_key_get_keytag(*key))) {
 			return KNOT_EOK;
 		}
+
+		(void)dnssec_keystore_remove(ctx->keystore, *id);
+		dnssec_key_free(*key);
+		free(*id);
 	}
 
 	log_zone_notice(ctx->zone->dname, "generated key with conflicting keytag %hu",
@@ -179,6 +181,7 @@ int kdnssec_generate_key(kdnssec_ctx_t *ctx, kdnssec_generate_flags_t flags,
 	r = kasp_zone_append(ctx->zone, key);
 	free(key);
 	if (r != KNOT_EOK) {
+		(void)dnssec_keystore_remove(ctx->keystore, id);
 		dnssec_key_free(dnskey);
 		free(id);
 		return r;
@@ -456,9 +459,9 @@ static int walk_algorithms(kdnssec_ctx_t *ctx, zone_keyset_t *keyset)
 /*!
  * \brief Load private keys for active keys.
  */
-static int load_private_keys(dnssec_keystore_t *keystore, zone_keyset_t *keyset)
+static int load_private_keys(kdnssec_ctx_t *ctx, zone_keyset_t *keyset)
 {
-	assert(keystore);
+	assert(ctx);
 	assert(keyset);
 
 	for (size_t i = 0; i < keyset->count; i++) {
@@ -466,17 +469,21 @@ static int load_private_keys(dnssec_keystore_t *keystore, zone_keyset_t *keyset)
 		if (!key->is_active && !key->is_ksk_active_plus && !key->is_zsk_active_plus) {
 			continue;
 		}
-		int r = dnssec_keystore_get_private(keystore, key->id, key->key);
-		switch (r) {
+		int ret = dnssec_keystore_get_private(ctx->keystore, key->id, key->key);
+		switch (ret) {
 		case DNSSEC_EOK:
 		case DNSSEC_KEY_ALREADY_PRESENT:
 			break;
 		default:
-			return r;
+			ret = knot_error_from_libdnssec(ret);
+			log_zone_error(ctx->zone->dname,
+			               "DNSSEC, key %d, failed to load private key (%s)",
+			               dnssec_key_get_keytag(key->key), knot_strerror(ret));
+			return ret;
 		}
 	}
 
-	return DNSSEC_EOK;
+	return KNOT_EOK;
 }
 
 /*!
@@ -566,11 +573,8 @@ int load_zone_keys(kdnssec_ctx_t *ctx, zone_keyset_t *keyset_ptr, bool verbose)
 		return ret;
 	}
 
-	ret = load_private_keys(ctx->keystore, &keyset);
-	ret = knot_error_from_libdnssec(ret);
+	ret = load_private_keys(ctx, &keyset);
 	if (ret != KNOT_EOK) {
-		log_zone_error(ctx->zone->dname, "DNSSEC, failed to load private "
-		               "keys (%s)", knot_strerror(ret));
 		free_zone_keys(&keyset);
 		return ret;
 	}
