@@ -17,6 +17,7 @@
 #define __APPLE_USE_RFC_3542
 
 #include <assert.h>
+#include <gnutls/x509.h>
 #include <sys/types.h>   // OpenBSD
 #include <netinet/tcp.h> // TCP_FASTOPEN
 #include <sys/resource.h>
@@ -24,9 +25,9 @@
 #include "libknot/libknot.h"
 #include "libknot/yparser/ypschema.h"
 #include "libknot/xdp.h"
-#if defined ENABLE_XDP && ENABLE_QUIC
-#include "libknot/xdp/quic.h"
-#endif // ENABLE_XDP && ENABLE_QUIC
+#ifdef ENABLE_QUIC
+#include "libknot/quic/quic.h"
+#endif // ENABLE_QUIC
 #include "knot/common/log.h"
 #include "knot/common/stats.h"
 #include "knot/common/systemd.h"
@@ -39,10 +40,13 @@
 #include "knot/server/server.h"
 #include "knot/server/udp-handler.h"
 #include "knot/server/tcp-handler.h"
+#include "knot/updates/acl.h"
 #include "knot/zone/timers.h"
 #include "knot/zone/zonedb-load.h"
 #include "knot/worker/pool.h"
+#include "contrib/base64.h"
 #include "contrib/conn_pool.h"
+#include "contrib/files.h"
 #include "contrib/net.h"
 #include "contrib/openbsd/strlcat.h"
 #include "contrib/os.h"
@@ -57,6 +61,8 @@
 #include <linux/filter.h>
 #endif
 
+#define SESSION_TICKET_POOL_TIMEOUT (24 * 3600)
+
 /*! \brief Minimal send/receive buffer sizes. */
 enum {
 	UDP_MIN_RCVSIZE = 4096,
@@ -192,9 +198,10 @@ static bool enable_pktinfo(int sock, int family)
 		break;
 	case AF_INET6:
 		level = IPPROTO_IPV6;
-		option = IPV6_RECVPKTINFO;
+		option = IPV6_RECVPKTINFO; /* Multiplatform */
 		break;
 	default:
+		assert(0);
 		return false;
 	}
 
@@ -227,6 +234,22 @@ static int disable_pmtudisc(int sock, int family)
 	return KNOT_EOK;
 }
 
+static size_t quic_rmt_count(conf_t *conf)
+{
+	size_t count = 0;
+
+	for (conf_iter_t iter = conf_iter(conf, C_RMT);
+	     iter.code == KNOT_EOK; conf_iter_next(conf, &iter)) {
+		conf_val_t id = conf_iter_id(conf, &iter);
+		conf_val_t rmt_quic = conf_id_get(conf, C_RMT, C_QUIC, &id);
+		if (conf_bool(&rmt_quic)) {
+			count++;
+		}
+	}
+
+	return count;
+}
+
 static iface_t *server_init_xdp_iface(struct sockaddr_storage *addr, bool route_check,
                                       bool udp, bool tcp, uint16_t quic, unsigned *thread_id_start)
 {
@@ -327,6 +350,7 @@ static iface_t *server_init_xdp_iface(struct sockaddr_storage *addr, bool route_
  * Both TCP and UDP sockets will be created for the interface.
  *
  * \param addr              Socket address.
+ * \param quic              QUIC interface indication.
  * \param udp_thread_count  Number of created UDP workers.
  * \param tcp_thread_count  Number of created TCP workers.
  * \param tcp_reuseport     Indication if reuseport on TCP is enabled.
@@ -335,7 +359,7 @@ static iface_t *server_init_xdp_iface(struct sockaddr_storage *addr, bool route_
  * \retval Pointer to a new initialized interface.
  * \retval NULL if error.
  */
-static iface_t *server_init_iface(struct sockaddr_storage *addr,
+static iface_t *server_init_iface(struct sockaddr_storage *addr, bool quic,
                                   int udp_thread_count, int tcp_thread_count,
                                   bool tcp_reuseport, bool socket_affinity)
 {
@@ -352,14 +376,14 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 
 	int udp_socket_count = 1;
 	int udp_bind_flags = 0;
-	int tcp_socket_count = 1;
+	int tcp_socket_count = !quic ? 1 : 0;
 	int tcp_bind_flags = 0;
 
 #ifdef ENABLE_REUSEPORT
 	udp_socket_count = udp_thread_count;
 	udp_bind_flags |= NET_BIND_MULTIPLE;
 
-	if (tcp_reuseport) {
+	if (!quic && tcp_reuseport) {
 		tcp_socket_count = tcp_thread_count;
 		tcp_bind_flags |= NET_BIND_MULTIPLE;
 	}
@@ -379,6 +403,7 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 	bool warn_cbpf = true;
 	bool warn_bufsize = true;
 	bool warn_pktinfo = true;
+	bool warn_ecn = true;
 	bool warn_flag_misc = true;
 
 	/* Create bound UDP sockets. */
@@ -400,7 +425,8 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 			return NULL;
 		}
 
-		if ((udp_bind_flags & NET_BIND_MULTIPLE) && socket_affinity) {
+		if ((udp_bind_flags & NET_BIND_MULTIPLE) && socket_affinity &&
+		    addr->ss_family != AF_UNIX) {
 			if (!server_attach_reuseport_bpf(sock, udp_socket_count) &&
 			    warn_cbpf) {
 				log_warning("cannot ensure optimal CPU locality for UDP");
@@ -414,10 +440,12 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 			warn_bufsize = false;
 		}
 
-		if (sockaddr_is_any(addr) && !enable_pktinfo(sock, addr->ss_family) &&
-		    warn_pktinfo) {
-			log_warning("failed to enable received packet information retrieval");
-			warn_pktinfo = false;
+		if (sockaddr_is_any(addr)) {
+			new_if->anyaddr = true;
+			if (!enable_pktinfo(sock, addr->ss_family) && warn_pktinfo) {
+				log_warning("failed to enable PKTINFO for ANY address interface");
+				warn_pktinfo = false;
+			}
 		}
 
 		int ret = disable_pmtudisc(sock, addr->ss_family);
@@ -427,6 +455,14 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 			warn_flag_misc = false;
 		}
 
+		if (quic) {
+			ret = net_cmsg_ecn_enable(sock, addr->ss_family);
+			if (ret != KNOT_EOK && ret != KNOT_ENOTSUP && warn_ecn) {
+				log_warning("failed to enable ECN for QUIC");
+				warn_ecn = false;
+			}
+		}
+
 		new_if->fd_udp[new_if->fd_udp_count] = sock;
 		new_if->fd_udp_count += 1;
 	}
@@ -472,7 +508,8 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 			return NULL;
 		}
 
-		if ((tcp_bind_flags & NET_BIND_MULTIPLE) && socket_affinity) {
+		if ((tcp_bind_flags & NET_BIND_MULTIPLE) && socket_affinity &&
+		    addr->ss_family != AF_UNIX) {
 			if (!server_attach_reuseport_bpf(sock, tcp_socket_count) &&
 			    warn_cbpf) {
 				log_warning("cannot ensure optimal CPU locality for TCP");
@@ -520,6 +557,82 @@ static void log_sock_conf(conf_t *conf)
 	}
 }
 
+#ifdef ENABLE_QUIC
+static int check_file(char *path, char *role)
+{
+	if (path == NULL) {
+		return KNOT_EOK;
+	}
+
+	char *err_str;
+
+	struct stat st;
+	int ret = stat(path, &st);
+	if (ret != 0) {
+		err_str = "invalid file";
+	} else if (!S_ISREG(st.st_mode)) {
+		err_str = "not a file";
+	} else {
+		return KNOT_EOK;
+	}
+
+	log_error("QUIC, %s file '%s' (%s)", role, path, err_str);
+	return KNOT_EINVAL;
+}
+#endif // ENABLE_QUIC
+
+static int init_creds(server_t *server, conf_t *conf)
+{
+#ifdef ENABLE_QUIC
+	char *cert_file = conf_tls(conf, C_CERT_FILE);
+	char *key_file = conf_tls(conf, C_KEY_FILE);
+
+	int ret = check_file(cert_file, "certificate");
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	ret = check_file(key_file, "key");
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	if (cert_file == NULL) {
+		assert(key_file == NULL);
+		char *kasp_dir = conf_db(conf, C_KASP_DB);
+		ret = make_dir(kasp_dir, S_IRWXU | S_IRWXG, true);
+		if (ret != KNOT_EOK) {
+			log_error("QUIC, failed to create directory '%s'", kasp_dir);
+			free(kasp_dir);
+			return ret;
+		}
+		key_file = abs_path(DFLT_QUIC_KEY_FILE, kasp_dir);
+		free(kasp_dir);
+		log_debug("QUIC, using self-generated key '%s' with "
+		          "one-time certificate", key_file);
+	}
+	server->quic_creds = knot_quic_init_creds(cert_file, key_file);
+	free(cert_file);
+	if (server->quic_creds == NULL) {
+		log_error("QUIC, failed to initialize server credentials with key '%s'",
+		          key_file);
+		free(key_file);
+		return KNOT_ERROR;
+	}
+	free(key_file);
+
+	size_t pin_len;
+	uint8_t pin[128];
+	if ((pin_len = server_cert_pin(server, pin, sizeof(pin))) > 0) {
+		log_info("QUIC, certificate public key %.*s", (int)pin_len, pin);
+	}
+
+	return KNOT_EOK;
+#else
+	return KNOT_ERROR;
+#endif // ENABLE_QUIC
+}
+
 /*! \brief Initialize bound sockets according to configuration. */
 static int configure_sockets(conf_t *conf, server_t *s)
 {
@@ -528,10 +641,12 @@ static int configure_sockets(conf_t *conf, server_t *s)
 	}
 
 	conf_val_t listen_val = conf_get(conf, C_SRV, C_LISTEN);
+	conf_val_t liquic_val = conf_get(conf, C_SRV, C_LISTEN_QUIC);
 	conf_val_t lisxdp_val = conf_get(conf, C_XDP, C_LISTEN);
 	conf_val_t rundir_val = conf_get(conf, C_SRV, C_RUNDIR);
+	uint16_t convent_quic = conf_val_count(&liquic_val);
 
-	if (listen_val.code == KNOT_EOK) {
+	if (listen_val.code == KNOT_EOK || liquic_val.code == KNOT_EOK) {
 		log_sock_conf(conf);
 	} else if (lisxdp_val.code != KNOT_EOK) {
 		log_warning("no network interface configured");
@@ -556,7 +671,8 @@ static int configure_sockets(conf_t *conf, server_t *s)
 #endif
 
 	size_t real_nifs = 0;
-	size_t nifs = conf_val_count(&listen_val) + conf_val_count(&lisxdp_val);
+	size_t nifs = conf_val_count(&listen_val) + conf_val_count(&liquic_val) +
+	              conf_val_count(&lisxdp_val);
 	iface_t *newlist = calloc(nifs, sizeof(*newlist));
 	if (newlist == NULL) {
 		log_error("failed to allocate memory for network sockets");
@@ -575,7 +691,7 @@ static int configure_sockets(conf_t *conf, server_t *s)
 		sockaddr_tostr(addr_str, sizeof(addr_str), &addr);
 		log_info("binding to interface %s", addr_str);
 
-		iface_t *new_if = server_init_iface(&addr, size_udp, size_tcp,
+		iface_t *new_if = server_init_iface(&addr, false, size_udp, size_tcp,
 		                                    tcp_reuseport, socket_affinity);
 		if (new_if == NULL) {
 			server_deinit_iface_list(newlist, nifs);
@@ -587,6 +703,25 @@ static int configure_sockets(conf_t *conf, server_t *s)
 
 		conf_val_next(&listen_val);
 	}
+	while (liquic_val.code == KNOT_EOK) {
+		struct sockaddr_storage addr = conf_addr(&liquic_val, rundir);
+		char addr_str[SOCKADDR_STRLEN] = { 0 };
+		sockaddr_tostr(addr_str, sizeof(addr_str), &addr);
+		log_info("binding to QUIC interface %s", addr_str);
+
+		iface_t *new_if = server_init_iface(&addr, true, size_udp, 0,
+		                                    false, socket_affinity);
+		if (new_if == NULL) {
+			server_deinit_iface_list(newlist, nifs);
+			free(rundir);
+			return KNOT_ERROR;
+		}
+		new_if->quic = true;
+		memcpy(&newlist[real_nifs++], new_if, sizeof(*newlist));
+		free(new_if);
+
+		conf_val_next(&liquic_val);
+	}
 	free(rundir);
 
 	/* XDP sockets. */
@@ -616,23 +751,14 @@ static int configure_sockets(conf_t *conf, server_t *s)
 	assert(real_nifs <= nifs);
 	nifs = real_nifs;
 
-#if defined ENABLE_XDP && ENABLE_QUIC
-	if (xdp_quic > 0) {
-		char *tls_cert = conf_tls(conf, C_CERT_FILE);
-		char *tls_key = conf_tls(conf, C_KEY_FILE);
-		if (tls_cert == NULL) {
-			log_notice("QUIC, no server certificate configured, using one-time one");
-		}
-		s->quic_creds = knot_xquic_init_creds(true, tls_cert, tls_key);
-		free(tls_cert);
-		free(tls_key);
-		if (s->quic_creds == NULL) {
-			log_error("QUIC, failed to initialize server credentials");
+	/* QUIC credentials initialization. */
+	s->quic_active = xdp_quic > 0 || convent_quic > 0 || quic_rmt_count(conf) > 0;
+	if (s->quic_active) {
+		if (init_creds(s, conf) != KNOT_EOK) {
 			server_deinit_iface_list(newlist, nifs);
 			return KNOT_ERROR;
 		}
 	}
-#endif // ENABLE_XDP && ENABLE_QUIC
 
 	/* Publish new list. */
 	s->ifaces = newlist;
@@ -748,11 +874,13 @@ void server_deinit(server_t *server)
 	/* Close and deinit connection pool. */
 	conn_pool_deinit(global_conn_pool);
 	global_conn_pool = NULL;
+	conn_pool_deinit(global_sessticket_pool);
+	global_sessticket_pool = NULL;
 	knot_unreachables_deinit(&global_unreachables);
 
-#if defined ENABLE_XDP && ENABLE_QUIC
-	knot_xquic_free_creds(server->quic_creds);
-#endif // ENABLE_XDP && ENABLE_QUIC
+#if defined ENABLE_QUIC
+	knot_quic_free_creds(server->quic_creds);
+#endif // ENABLE_QUIC
 }
 
 static int server_init_handler(server_t *server, int index, int thread_count,
@@ -916,14 +1044,16 @@ static int reload_conf(conf_t *new_conf)
 	return KNOT_EOK;
 }
 
-/*! \brief Check if parameter listen(-xdp) has been changed since knotd started. */
+/*! \brief Check if parameter listen(-xdp,-quic) has been changed since knotd started. */
 static bool listen_changed(conf_t *conf, server_t *server)
 {
 	assert(server->ifaces);
 
 	conf_val_t listen_val = conf_get(conf, C_SRV, C_LISTEN);
+	conf_val_t liquic_val = conf_get(conf, C_SRV, C_LISTEN_QUIC);
 	conf_val_t lisxdp_val = conf_get(conf, C_XDP, C_LISTEN);
-	size_t new_count = conf_val_count(&listen_val) + conf_val_count(&lisxdp_val);
+	size_t new_count = conf_val_count(&listen_val) + conf_val_count(&liquic_val) +
+	                   conf_val_count(&lisxdp_val);
 	size_t old_count = server->n_ifaces;
 	if (new_count != old_count) {
 		return true;
@@ -949,6 +1079,21 @@ static bool listen_changed(conf_t *conf, server_t *server)
 		}
 		conf_val_next(&listen_val);
 	}
+	while (liquic_val.code == KNOT_EOK) {
+		struct sockaddr_storage addr = conf_addr(&liquic_val, rundir);
+		bool found = false;
+		for (size_t i = 0; i < server->n_ifaces; i++) {
+			if (sockaddr_cmp(&addr, &server->ifaces[i].addr, false) == 0) {
+				matches++;
+				found = true;
+				break;
+			}
+		}
+		if (!found) {
+			break;
+		}
+		conf_val_next(&liquic_val);
+	}
 	free(rundir);
 
 	while (lisxdp_val.code == KNOT_EOK) {
@@ -1013,7 +1158,7 @@ static void warn_server_reconfigure(conf_t *conf, server_t *server)
 	}
 
 	if (warn_listen && server->ifaces != NULL && listen_changed(conf, server)) {
-		log_warning(msg, "listen(-xdp)");
+		log_warning(msg, "listen(-xdp,-quic)");
 		warn_listen = false;
 	}
 
@@ -1214,14 +1359,25 @@ static int reconfigure_timer_db(conf_t *conf, server_t *server)
 	return ret;
 }
 
-static int reconfigure_remote_pool(conf_t *conf)
+#ifdef ENABLE_QUIC
+static void free_sess_ticket(intptr_t ptr)
+{
+	if (ptr != CONN_POOL_FD_INVALID) {
+		knot_quic_session_load(NULL, (void *)ptr);
+	}
+}
+#endif // ENABLE_QUIC
+
+static int reconfigure_remote_pool(conf_t *conf, server_t *server)
 {
 	conf_val_t val = conf_get(conf, C_SRV, C_RMT_POOL_LIMIT);
 	size_t limit = conf_int(&val);
 	val = conf_get(conf, C_SRV, C_RMT_POOL_TIMEOUT);
 	knot_timediff_t timeout = conf_int(&val);
 	if (global_conn_pool == NULL && limit > 0) {
-		conn_pool_t *new_pool = conn_pool_init(limit, timeout);
+		conn_pool_t *new_pool = conn_pool_init(limit, timeout,
+		                                       conn_pool_close_cb_dflt,
+		                                       conn_pool_invalid_cb_dflt);
 		if (new_pool == NULL) {
 			return KNOT_ENOMEM;
 		}
@@ -1230,6 +1386,22 @@ static int reconfigure_remote_pool(conf_t *conf)
 		(void)conn_pool_timeout(global_conn_pool, timeout);
 	}
 
+#ifdef ENABLE_QUIC
+	if (global_sessticket_pool == NULL && server->quic_active) {
+		size_t rmt_count = quic_rmt_count(conf);
+		if (rmt_count > 0) {
+			size_t max_tickets = conf_bg_threads(conf) * rmt_count * 2; // Two addresses per remote.
+			conn_pool_t *new_pool =
+				conn_pool_init(max_tickets, SESSION_TICKET_POOL_TIMEOUT,
+				               free_sess_ticket, conn_pool_invalid_cb_allvalid);
+			if (new_pool == NULL) {
+				return KNOT_ENOMEM;
+			}
+			global_sessticket_pool = new_pool;
+		}
+	}
+#endif // ENABLE_QUIC
+
 	val = conf_get(conf, C_SRV, C_RMT_RETRY_DELAY);
 	int delay_ms = conf_int(&val);
 	if (global_unreachables == NULL && delay_ms > 0) {
@@ -1298,7 +1470,7 @@ int server_reconfigure(conf_t *conf, server_t *server)
 	}
 
 	/* Reconfigure connection pool. */
-	if ((ret = reconfigure_remote_pool(conf)) != KNOT_EOK) {
+	if ((ret = reconfigure_remote_pool(conf, server)) != KNOT_EOK) {
 		log_error("failed to reconfigure remote pool (%s)",
 		          knot_strerror(ret));
 	}
@@ -1333,3 +1505,25 @@ void server_update_zones(conf_t *conf, server_t *server, reload_t mode)
 		knot_zonedb_foreach(server->zone_db, zone_events_start);
 	}
 }
+
+size_t server_cert_pin(server_t *server, uint8_t *out, size_t out_size)
+{
+#ifdef ENABLE_QUIC
+	int pin_size = 0;
+
+	uint8_t bin_pin[KNOT_QUIC_PIN_LEN];
+	size_t bin_pin_size = sizeof(bin_pin);
+	gnutls_x509_crt_t cert = NULL;
+	if (server->quic_creds != NULL &&
+	    knot_quic_creds_cert(server->quic_creds, &cert) == KNOT_EOK &&
+	    gnutls_x509_crt_get_key_id(cert, GNUTLS_KEYID_USE_SHA256,
+	                               bin_pin, &bin_pin_size) == GNUTLS_E_SUCCESS) {
+		pin_size = knot_base64_encode(bin_pin, bin_pin_size, out, out_size);
+	}
+	gnutls_x509_crt_deinit(cert);
+
+	return (pin_size >= 0) ? pin_size : 0;
+#else
+	return 0;
+#endif // ENABLE_QUIC
+}
