@@ -379,6 +379,31 @@ done:
 	return ret;
 }
 
+static int backup_kaspdb(zone_backup_ctx_t *ctx, conf_t *conf, zone_t *zone,
+                         int (*kaspdb_backup_cb)(const knot_dname_t *zone,
+                                                 knot_lmdb_db_t *db, knot_lmdb_db_t *backup_db))
+{
+	knot_lmdb_db_t *kasp_from = zone_kaspdb(zone), *kasp_to = &ctx->bck_kasp_db;
+	BACKUP_SWAP(ctx, kasp_from, kasp_to);
+
+	if (knot_lmdb_exists(kasp_from) != KNOT_ENODB) {
+		int ret = kaspdb_backup_cb(zone->name, kasp_from, kasp_to);
+		if (ret != KNOT_EOK) {
+			LOG_MARK_FAIL("KASP database");
+			return ret;
+		}
+
+		ret = backup_keystore(conf, zone, ctx);
+		if (ret != KNOT_EOK) {
+			// Errors already logged in detail.
+			ctx->failed = true;
+			return ret;
+		}
+	}
+
+	return KNOT_EOK;
+}
+
 int zone_backup(conf_t *conf, zone_t *zone)
 {
 	zone_backup_ctx_t *ctx = zone->backup_ctx;
@@ -387,32 +412,20 @@ int zone_backup(conf_t *conf, zone_t *zone)
 	}
 
 	int ret = KNOT_EOK;
-	int ret_deinit;
 
 	if (ctx->backup_zonefile) {
 		ret = backup_zonefile(conf, zone, ctx);
 		if (ret != KNOT_EOK) {
 			LOG_MARK_FAIL("zone file");
-			goto done;
+			return ret;
 		}
 	}
 
 	if (ctx->backup_kaspdb) {
-		knot_lmdb_db_t *kasp_from = zone_kaspdb(zone), *kasp_to = &ctx->bck_kasp_db;
-		BACKUP_SWAP(ctx, kasp_from, kasp_to);
-
-		if (knot_lmdb_exists(kasp_from) != KNOT_ENODB) {
-			ret = kasp_db_backup(zone->name, kasp_from, kasp_to);
-			if (ret != KNOT_EOK) {
-				LOG_MARK_FAIL("KASP database");
-				goto done;
-			}
-
-			ret = backup_keystore(conf, zone, ctx);
-			if (ret != KNOT_EOK) {
-				ctx->failed = true;
-				goto done;
-			}
+		ret = backup_kaspdb(ctx, conf, zone, kasp_db_backup);
+		if (ret != KNOT_EOK) {
+			// Errors already logged in detail.
+			return ret;
 		}
 	}
 
@@ -426,14 +439,14 @@ int zone_backup(conf_t *conf, zone_t *zone)
 	}
 	if (ret != KNOT_EOK) {
 		LOG_MARK_FAIL("journal");
-		goto done;
+		return ret;
 	}
 
 	if (ctx->backup_timers) {
 		ret = knot_lmdb_open(&ctx->bck_timer_db);
 		if (ret != KNOT_EOK) {
 			LOG_MARK_FAIL("timers open");
-			goto done;
+			return ret;
 		}
 		if (ctx->restore_mode) {
 			ret = zone_timers_read(&ctx->bck_timer_db, zone->name, &zone->timers);
@@ -444,14 +457,11 @@ int zone_backup(conf_t *conf, zone_t *zone)
 		}
 		if (ret != KNOT_EOK) {
 			LOG_MARK_FAIL("timers");
-			goto done;
+			return ret;
 		}
 	}
 
-done:
-	ret_deinit = zone_backup_deinit(ctx);
-	zone->backup_ctx = NULL;
-	return (ret != KNOT_EOK) ? ret : ret_deinit;
+	return ret;
 }
 
 int global_backup(zone_backup_ctx_t *ctx, catalog_t *catalog,
@@ -470,6 +480,11 @@ int global_backup(zone_backup_ctx_t *ctx, catalog_t *catalog,
 	return ret;
 }
 
+int zone_backup_keysonly(zone_backup_ctx_t *ctx, conf_t *conf, zone_t *zone)
+{
+	return backup_kaspdb(ctx, conf, zone, kasp_db_backup_keys);
+}
+
 static int backup_quic_file(zone_backup_ctx_t *ctx, char *file, char *subdir,
                             const char *desc, bool required, bool *success)
 {
