@@ -1,4 +1,4 @@
-/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2023 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -51,6 +51,15 @@ typedef enum {
 /*! \brief Context structure for one XDP socket. */
 typedef struct knot_xdp_socket knot_xdp_socket_t;
 
+/*! \brief Configuration of XDP socket. */
+struct knot_xdp_config {
+	bool force_generic; /*!< Use generic XDP mode (avoid driver/hadrware implementation). */
+	bool force_copy;    /*!< Force copying packet data between kernel and user-space (avoid zero-copy). */
+};
+
+/*! \brief Configuration of XDP socket. */
+typedef struct knot_xdp_config knot_xdp_config_t;
+
 /*!
  * \brief Initialize XDP socket.
  *
@@ -67,7 +76,7 @@ typedef struct knot_xdp_socket knot_xdp_socket_t;
  */
 int knot_xdp_init(knot_xdp_socket_t **socket, const char *if_name, int if_queue,
                   knot_xdp_filter_flag_t flags, uint16_t udp_port, uint16_t quic_port,
-                  knot_xdp_load_bpf_t load_bpf, const void *xdp_config);
+                  knot_xdp_load_bpf_t load_bpf, const knot_xdp_config_t *xdp_config);
 
 /*!
  * \brief De-init XDP socket.
@@ -184,6 +193,6 @@ void knot_xdp_recv_finish(knot_xdp_socket_t *socket, const knot_xdp_msg_t msgs[]
  * \param socket  XDP socket.
  * \param file    Output file.
  */
-void knot_xdp_info(const knot_xdp_socket_t *socket, FILE *file);
+void knot_xdp_socket_info(const knot_xdp_socket_t *socket, FILE *file);
 
 /*! @} */
