@@ -1,4 +1,4 @@
-/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2023 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #include <stdlib.h>
 
 #include "utils/kdig/kdig_params.h"
-#include "utils/common/cert.h"
 #include "utils/common/hex.h"
 #include "utils/common/msg.h"
 #include "utils/common/netio.h"
@@ -433,6 +432,24 @@ static int opt_noquestion(const char *arg, void *query)
 	return KNOT_EOK;
 }
 
+static int opt_optpresent(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	q->style.present_edns = true;
+
+	return KNOT_EOK;
+}
+
+static int opt_nooptpresent(const char *arg, void *query)
+{
+	query_t *q = query;
+
+	q->style.present_edns = false;
+
+	return KNOT_EOK;
+}
+
 static int opt_answer(const char *arg, void *query)
 {
 	query_t *q = query;
@@ -655,6 +672,9 @@ static int opt_tls(const char *arg, void *query)
 	query_t *q = query;
 
 	q->tls.enable = true;
+	if (q->quic.enable) {
+		return KNOT_EOK;
+	}
 	return opt_tcp(arg, query);
 }
 
@@ -1511,6 +1531,9 @@ static const param_t kdig_opts2[] = {
 	{ "opttext",        ARG_NONE,     opt_opttext },
 	{ "noopttext",      ARG_NONE,     opt_noopttext },
 
+	{ "optpresent",     ARG_NONE,     opt_optpresent },
+	{ "nooptpresent",   ARG_NONE,     opt_nooptpresent },
+
 	{ "question",       ARG_NONE,     opt_question },
 	{ "noquestion",     ARG_NONE,     opt_noquestion },
 
@@ -2321,6 +2344,7 @@ static void print_help(void)
 	       "       +[no]comments              Show commented section names.\n"
 	       "       +[no]opt                   Show EDNS pseudosection.\n"
 	       "       +[no]opttext               Try to show unknown EDNS options as text.\n"
+	       "       +[no]optpresent            Show EDNS in presenatation format.\n"
 	       "       +[no]question              Show question section.\n"
 	       "       +[no]answer                Show answer section.\n"
 	       "       +[no]authority             Show authority section.\n"
