@@ -42,15 +42,17 @@ signal_ctx_t signal_ctx = { 0 }; // global, needed by signal handler
 static void print_help(void)
 {
 	printf("Usage:\n"
-	       " %s [-c | -C | -D <path>] [parameters] <zone_name>\n"
+	       " %s [-c | -C | -D <path>] [options] <zone_name>\n"
 	       " %s [-c | -C | -D <path>] -z\n"
 	       "\n"
-	       "Parameters:\n"
+	       "Config options:\n"
 	       " -c, --config <file>  Path to a textual configuration file.\n"
 	       "                       (default %s)\n"
 	       " -C, --confdb <dir>   Path to a configuration database directory.\n"
 	       "                       (default %s)\n"
-	       " -D, --dir <path>     Path to a journal database directory, use default configuration.\n"
+	       " -D, --dir <path>     Path to a journal database directory, use default\n"
+	       "                      configuration.\n"
+	       "Options:\n"
 	       " -z, --zone-list      Instead of reading the journal, display the list\n"
 	       "                      of zones in the DB.\n"
 	       " -l, --limit <num>    Read only <num> newest changes.\n"
